// Programmed using the knockoff Pro Micro boards - use Arduino/Genuino Micro as board type
// Watch out - your board's name might change and you'll have to seelct it again in Tools/Port 
// after you program it the first time

#include <Wire.h>

// Libraries 
// You need to install all three libraries - 
// Search for them under Sketch/Include Library/Manage Libraries, 
// Then search for the word in front of the .h
#include <elapsedMillis.h>
#include <MIDIUSB.h>
#include "Adafruit_MPR121.h"

// Define some bit-wise operations to be used later
#ifndef _BV
#define _BV(bit) (1 << (bit)) 
#endif

// Variables used to calculate centroid value
float centroidArea = 0;
float area = 0;
float prevCentroid = 0;
float currCentroid = 0;
float pressure = 0;

// Keeps track of the last pins touched
// so we know when buttons are 'released'
uint16_t lasttouched = 0;
uint16_t currtouched = 0;

// A variable used to store initial sensor readings as part of calibration
int cal[11];

// Initialize the capacitive touch board
Adafruit_MPR121 cap = Adafruit_MPR121();

// Start a timer to be used to avoid overloading the MIDI bus with messages
elapsedMillis timeElapsed = 0;

// MIDI Information
int midiChannel = 0; // Midi channels are zero-indexed, so 0 = MIDI Channel 1
int midiCC = 100; // CC # to be used to send CC messages
int noteVelocity = 100; // Velocity at which the note is started 0-127 valued
int noteNum = 60; // Midi note number that gets played - 60 is middle C

// The setup loop will run once to intialize the hardware
void setup() {  
  // Default address is 0x5A,
  if (!cap.begin(0x5A)) {
    while (1);
  }
  
  // Wait a little bit before reading each pin and storing a calibration value
  delay(100);
  for(uint8_t i=0; i <= 10; i++){
    cal[i] = cap.filteredData(i);
  }
}

// The main loop repeats over and over...
void loop() {
  // Read from the board to see which pins are touched
  currtouched = cap.touched();
  
  // if any pin *is* touched and all pins were not touched before, send note on
  if ((currtouched) && !(lasttouched) ) {
    checkPosition();
    noteOn(midiChannel,noteNum,noteVelocity);
  }
  // if any pin *was* touched and now all pins are not, send note off
  if (!(currtouched) && (lasttouched)) {
    noteOff(midiChannel,noteNum,0);
  }
  // if any pin *was* touched and now any pin is touched, just update position
  if ((currtouched) && (lasttouched)) {
    checkPosition();
  }
  
  // reset our state for the next loop
  lasttouched = currtouched;
}

// A function to check the position where the strip is touched
void checkPosition() {

  // Reset variables to zero before doing the calculation
  centroidArea = 0;
  area = 0;
  float f = 0;
  float count = 0;
  
  // Cycle through each pin, read its value, and add it to the centroid calculation
  for(uint8_t i=0; i <= 10; i++){
    if (currtouched & _BV(i)){
      count++;
      f = cal[i]-cap.filteredData(i);    
    } else {
      f = 0;
    }
    centroidArea = centroidArea + i*pow(f, 5);
    area = area + pow(f, 5); 
  }

  // Update the new centroid value
  prevCentroid = currCentroid;
  currCentroid = centroidArea/area;

  // If none of the pins are touched, you get a divide by zero error
  // Handle that situation so things don't go wacky!
  if(area == 0){
    currCentroid = prevCentroid;  
    pressure = 0;
  }
  
  // Only send Pitch Bend at most 100 times per second
  if(timeElapsed > 10){
    int pbVal = map(10000*currCentroid, 0, 110000, 0, 16383);
    pitchBend(midiChannel, pbVal);    
  }
}

// A function to make it easier to send control change messages
void sendCC(byte channel, byte CC, byte value) {
  midiEventPacket_t CCdata = {0x0B, 0xB0 | channel, CC, value};
  MidiUSB.sendMIDI(CCdata);
  MidiUSB.flush();
}

// A function to make it easier to send Note On messages
void noteOn(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOn = {0x09, 0x90 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOn);
  MidiUSB.flush();
}

// A function to make it easier to send Note Off messages
void noteOff(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOff = {0x08, 0x80 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOff);
  MidiUSB.flush();
}

// A function to make it easier to send Pitch Bend messages
// The pitch bend value is a 14-bit number (0-16383). 0x2000 (8192) is the default / middle value.
void pitchBend(byte channel, int value) {
  byte lowValue = value & 0x7F;
  byte highValue = value >> 7;
  midiEventPacket_t event = {0x0E, 0xE0 | channel, lowValue, highValue};
  MidiUSB.sendMIDI(event);
  MidiUSB.flush();
}

// We used code from a library that needs this text included
/*********************************************************
This is a library for the MPR121 12-channel Capacitive touch sensor

Designed specifically to work with the MPR121 Breakout in the Adafruit shop 
  ----> https://www.adafruit.com/products/

These sensors use I2C communicate, at least 2 pins are required 
to interface

Adafruit invests time and resources providing this open source code, 
please support Adafruit and open-source hardware by purchasing 
products from Adafruit!

Written by Limor Fried/Ladyada for Adafruit Industries.  
BSD license, all text above must be included in any redistribution
**********************************************************/
